/***************************************************************************
                          calibrate.c  -  description
                             -------------------
    begin                : Sat Dec 25 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"

/*This file contains calibration and image-quality control functions */

/*--------------------------------------------------------------------------*/
//This function adjusts an led-on-time value during the first stage of
//the calibration sequence.


inline unsigned short fixmaxtime(unsigned char maxbright,
				 unsigned char brightval,
				 unsigned short ledtime)
{

    unsigned char adjust = 4, tolerance = 5;

    if (MODE == NIBBLE_MODE)
	tolerance = 7;

    if ((maxbright - brightval) > tolerance + 11)
	adjust = 6;
    if ((maxbright - brightval) < tolerance + 4)
	adjust = 2;
    if ((maxbright - brightval) < tolerance + 2)
	adjust = 1;

    if (brightval < (maxbright - tolerance)) {
	ledtime += adjust;	//printf("\nadjusting value"); fflush(stdout);
	return ledtime;
    }

    else if (brightval > maxbright) {
	ledtime -= adjust;	//printf("\nadjusting value"); fflush(stdout);
	return ledtime;
    }
    return ledtime;		//printf("\nreturning good value"); fflush(stdout);
}

/*--------------------------------------------------------------------------*/

inline unsigned short fixmintime(unsigned char minbright,
				 unsigned char brightval,
				 unsigned short ledtime)
{

    unsigned char adjust = 4, tolerance = 2;

    if (MODE == NIBBLE_MODE)
	tolerance = 7;

    if ((minbright - brightval) > tolerance + 11)
	adjust = 6;
    if ((minbright - brightval) < tolerance + 4)
	adjust = 2;
    if ((minbright - brightval) < tolerance + 2)
	adjust = 1;


    if (brightval < (minbright - tolerance)) {
	ledtime += adjust;	//printf("\nadjusting value"); fflush(stdout);
	return ledtime;
    }

    else if (brightval > minbright) {
	ledtime -= adjust;	//printf("\nadjusting value"); fflush(stdout);
	return ledtime;
    }
    return ledtime;		//printf("\nreturning good value"); fflush(stdout);
}

/*--------------------------------------------------------------------------*/
//waits for the motor to reach home position, one must be very careful
//though, as the scanner starts up with this flag OFF, and with very short
//scans, it may not record the motor as ever having left the home position
//and hence never resets the flag

void wait_home(int port)
{

    unsigned char data;

    for (;;) {
	data = read_register(port, status_control_register);
	if ((data & 0x20) == 0)
	    usleep(10000);
	else
	    break;
    }
}

/*--------------------------------------------------------------------------*/
//gets the values returned by the sensors for 0 light.  Suffers from light
//leakage, and probably can be changed from a moving to a stationary scan
//without harm.

void get_dark_values(int port, unsigned char resolution,
		     unsigned char color, unsigned short startpos,
		     unsigned short stoppos)
{
//FILE *dvd;
    int redsum[2550], greensum[2550], bluesum[2550];
    unsigned char redbuffer[2550], greenbuffer[2550], bluebuffer[2550];
    unsigned short lines, max_lines, bytes_per_line =
	0, readylines, needed = 1;
    unsigned short startline, stopline, setting;
    unsigned short linetime, minlinetime = 3300, motorspeed =
	0xc350, writelength;
    int pause = 0, dpi = 50, bytecounter, waitcounter, sleepcounter =
	0, linecounter;
    long totredsum = 0, totgreensum = 0, totbluesum = 0;

//dvd = fopen("darkvals","w+");
    fprintf(LOGd, "\nGetting	Dark Values");
    fflush(LOGd);
    if (resolution == DPI300)
	dpi = 300;
    if (resolution == DPI200)
	dpi = 200;
    if (resolution == DPI100)
	dpi = 100;
    if (resolution == DPI50)
	dpi = 50;

    writelength = ((stoppos * dpi / 300) - (startpos * dpi / 300));
    if (color == COLOR) {
	writelength = (3 * bytes_per_line);
	needed = 3;
    }
    //if (color == LINEART) writelength = (bytes_per_line + 7) /8;
    if ((color == COLOR) && (dpi == 50))
	minlinetime = 5000;

    linetime = (writelength * 3) / ( /*(MODE / 0x10) + */ 1);
    if (linetime < minlinetime) {
	linetime = minlinetime;
	pause = 1;
    }
    if (linetime > 8191)
	linetime = 8191;
    if ((color == GREYSCALE) && (dpi < 200)) {
	linetime = 8191;
	pause = 1;
    }
    fprintf(LOGd, "\nLinetime = %d", linetime);
    if ((color != COLOR) && (dpi < 200)) {
	linetime = 8191;
	pause = 1;
    }
    fprintf(LOGd, "Pause = %d", pause);
    if (MODE == EPP_MODE)
	linetime = 8191;

    bytes_per_line = (2550 * dpi) / 300;

    max_lines = 2700 / dpi;	//this can probably be stretched-- hack on 300 dpi

    memset(redsum, 0, sizeof(redsum));	//initialize the arrays
    memset(greensum, 0, sizeof(greensum));
    memset(bluesum, 0, sizeof(bluesum));

    startline = (2 * 300) / dpi;
    stopline = (10 * 300) / dpi;
    lines = 9 * dpi / 300;

    write_register(port, primary_register, MODE);
    write_register(port, scan_mode_register, (resolution | color));	//scan mode
    write_register(port, darkhead_register, 0x00);	//ignore no pixels
    write_register(port, lineart_threshold_up, 0x00);
    write_register(port, lineart_threshold_down, 0x00);
    write_register(port, red_led_on_time, 0);
    write_register(port, green_led_on_time, 0);
    write_register(port, blue_led_on_time, 0);
    write_register(port, configure_setting_register, 0x0);
    write_register(port, led_control, 0x00);	//hardware control
    write_register(port, line_time_register, linetime);
    write_register(port, motor_rotation_speed, motorspeed);
    write_register(port, image_pixel_start, 0);	//start at the left edge
    write_register(port, image_pixel_stop, 2550);	//end at the right edge
    write_register(port, empty_image_line_register, startline);	//Start at 3d line
    write_register(port, motor_steps_register, stopline);
    write_register(port, max_lines_register, max_lines);
    write_register(port, bytes_per_line_register, bytes_per_line);
    usleep(100000);

    for (waitcounter = 0; waitcounter < 10000; waitcounter++) {

	setting = read_register(port, configure_setting_register);
	if (setting == 0) {
	    write_register(port, primary_register, (ENABLE_SCAN | MODE));
	    break;
	}
	usleep(500);
	if (waitcounter > 9990) {
	    fprintf(stderr,
		    "We've got a problem, please see http://as6edriver.sourceforge.net\n");
	    test12(port);
	}
    }
    for (linecounter = 0; linecounter < lines; linecounter++) {
	readylines = read_register(port, buffer_line_count_register);
//printf("\nredylines = %d  sleepcounter = %d  linecounter = %d"
//,readylines,sleepcounter,linecounter);  fflush(stdout);
	if (readylines < needed) {
	    linecounter--;
	    sleepcounter++;
	    usleep(5);
	    if ((sleepcounter > 300) && (color == GREYSCALE)
			&& (readylines	 ==  (needed - 1))) {
		needed--;
		lines--;
	    }
	    if ((sleepcounter > 300) && (color == COLOR)
			&& (readylines == (needed - 3))) {
		needed -= 3;
		lines--;
	    }
	    if (sleepcounter > 1000) {
		fprintf(stderr, "\nDarkvalues timed out");
		exit(-1);
	    }
	} else {
	    readbuf(port, redbuffer, bytes_per_line);
	    if (color == COLOR) {
		readbuf(port, greenbuffer, bytes_per_line);
		readbuf(port, bluebuffer, bytes_per_line);
	    }
	    if (pause != 0)
		usleep(5);

	    for (bytecounter = 0; bytecounter < bytes_per_line;
		 bytecounter++) {
		redsum[bytecounter] += redbuffer[bytecounter];
		totredsum += redbuffer[bytecounter];
		if (color == COLOR) {
		    greensum[bytecounter] += greenbuffer[bytecounter];
		    totgreensum += greenbuffer[bytecounter];
		    bluesum[bytecounter] += bluebuffer[bytecounter];
		    totbluesum += bluebuffer[bytecounter];
		}		//end color
	    }			//end bytecounter
	}			//end else
    }				//end linecounter
    write_register(port, primary_register, MODE);

    for (bytecounter = 0; bytecounter < bytes_per_line; bytecounter++) {
	red_dark[bytecounter] = totredsum / (bytes_per_line * 8);	/*redsum[bytecounter] / 8; */
	//fprintf(dvd,"\n byte %d red 0x%x",bytecounter,red_dark[bytecounter]);
	if (color == COLOR) {
	    green_dark[bytecounter] = totgreensum / (bytes_per_line * 8);
//fprintf(dvd,"\nbyte %d green 0x%x",bytecounter,green_dark[bytecounter]);
	    blue_dark[bytecounter] = totbluesum / (bytes_per_line * 8);
	}			//end color
    }
//fclose(dvd);
    fprintf(LOGd, "\nDone Getting Dark Values");
    fflush(LOGd);
    return;
}

/*--------------------------------------------------------------------------*/
//Scans the supposedly unifor white strip at the front of the scanner
//to get sensor sensitivity information-- this varies aby almost 50%!

void get_light_values(int port, unsigned char resolution,
		      unsigned char color, unsigned short startpos,
		      unsigned short stoppos, unsigned char *redbuf,
		      unsigned char *greenbuf, unsigned char *bluebuf)
{
//    FILE *lvd;
    int redsum[2550], greensum[2550], bluesum[2550], linecounter;
    unsigned char redbuffer[2550], greenbuffer[2550], bluebuffer[2550];
    unsigned short max_lines, bytes_per_line, readylines, needed =
	1, startline, stopline;
    unsigned short linetime, minlinetime = 3300, motorspeed =
	0xc350, writelength, setting;
    int pause = 0, dpi =
	50, bytecounter, waitcounter, numlines, sleepcounter = 0;

//teststrip = fopen("lightstrip.pnm","w+");

//    lvd = fopen("lightval","w+");
    fprintf(LOGd, "\nGetting Light Values");
    fflush(LOGd);
    if (resolution == DPI300)
	dpi = 300;
    if (resolution == DPI200)
	dpi = 200;
    if (resolution == DPI100)
	dpi = 100;
    if (resolution == DPI50)
	dpi = 50;

    startline = 0;
    stopline = 48;		//this is about as far as it is safe to go-- on my scanner
    //the strip is slightly askew.
    numlines = ((49 * dpi) / 300);

    writelength = bytes_per_line =
	((stoppos * dpi / 300) - (startpos * dpi / 300));
    if (color == COLOR) {
	writelength = (3 * bytes_per_line);
	needed = 3;
    }
    //if (color == LINEART) writelength = (bytes_per_line + 7) /8;
    if ((color == COLOR) && (dpi == 50))
	minlinetime = 5000;


    linetime = (writelength * 3) / ( /*(MODE / 0x10) + */ 1);
    if (linetime < minlinetime) {
	linetime = minlinetime;
	pause = 1;
    }
    if (linetime > 8191)
	linetime = 8191;
    if ((color == GREYSCALE) && (dpi < 200))
	linetime = 8191;
    fprintf(LOGd, "\nLinetime = %d", linetime);
    if ((color != COLOR) && (dpi < 200))
	pause = 1;
    if (MODE == EPP_MODE)
	linetime = 8191;

    max_lines = 2700 / dpi;	//this can probably be stretched-- hack on 300 dpi

    memset(redsum, 0, sizeof(redsum));	//initialize the arrays
    memset(greensum, 0, sizeof(greensum));
    memset(bluesum, 0, sizeof(bluesum));

    write_register(port, primary_register, MODE);
    write_register(port, red_led_on_time, redtime);
    write_register(port, green_led_on_time, greentime);
    write_register(port, blue_led_on_time, bluetime);
    write_register(port, configure_setting_register, 0x0);
    write_register(port, line_time_register, linetime);
    write_register(port, motor_rotation_speed, motorspeed);
    write_register(port, empty_image_line_register, startline);
    write_register(port, motor_steps_register, stopline);
    write_register(port, max_lines_register, max_lines);
    write_register(port, bytes_per_line_register, bytes_per_line);
    usleep(100000);
    if (MODE == EPP_MODE)
	usleep(300000);

    for (waitcounter = 0; waitcounter < 10000; waitcounter++) {

	setting = read_register(port, configure_setting_register);
	if (setting == 0) {
	    write_register(port, primary_register, (ENABLE_SCAN | MODE));
	    break;
	}
	usleep(500);
	if (waitcounter > 9990) {
	    fprintf(stderr,
		    "We've got a problem, please see http://as6edriver.sourceforge.net\n");
	    fflush(stdout);
	    test12(port);
	}
    }
    for (linecounter = 0; linecounter < numlines; linecounter++) {

	readylines = read_register(port, buffer_line_count_register);
//printf("\nredylines = %d  sleepcounter = %d  linecounter = %d",readylines
//,sleepcounter,linecounter);  fflush(stdout);
	if (readylines < needed) {
	    linecounter--;
	    sleepcounter++;

	    usleep(5);
	    if ((sleepcounter > 300) && (color == GREYSCALE)
			&& (readylines	 == (needed - 1))) {
		needed--;
		numlines--;
	    }
	    if ((sleepcounter > 300) && (color == COLOR)
			&& (readylines == (needed - 3))) {
		needed -= 3;
		numlines--;
	    }
	    if (sleepcounter > 3000) {
		fprintf(stderr, "\nLightvalues timed out");
		exit(-1);
	    }
	} else {
	    readbuf(port, redbuffer, bytes_per_line);
	    if (color == COLOR) {
		readbuf(port, greenbuffer, bytes_per_line);
		readbuf(port, bluebuffer, bytes_per_line);
	    }
	    for (bytecounter = 0; bytecounter < bytes_per_line;
		 bytecounter++) {
//fwrite(&redbuffer[bytecounter],1,1,teststrip);
		redsum[bytecounter] += redbuffer[bytecounter];
		if (color == COLOR) {
//fwrite(&greenbuffer[bytecounter],1,1,teststrip);
		    greensum[bytecounter] += greenbuffer[bytecounter];
//fwrite(&bluebuffer[bytecounter],1,1,teststrip);
		    bluesum[bytecounter] += bluebuffer[bytecounter];
		}		//end color
	    }			//end bytecounter
	    usleep(5);
	}			//end else
    }				//end linecounter

    write_register(port, primary_register, MODE);

    for (bytecounter = 0; bytecounter < bytes_per_line; bytecounter++) {
	redbuf[bytecounter] = redsum[bytecounter] / numlines;

//fprintf(lvd,"\nbyte %d red 0x%x",bytecounter,redbuf[bytecounter]);
	if (color == COLOR) {
	    greenbuf[bytecounter] = greensum[bytecounter] / numlines;

//fprintf(lvd,"\nbyte %d green 0x%x",bytecounter,greenbuf[bytecounter]);
	    bluebuf[bytecounter] = bluesum[bytecounter] / numlines;

//fprintf(lvd,"\nbyte %d blue 0x%x",bytecounter,bluebuf[bytecounter]);
	}			//end color
    }				//end bytes per line
//fclose(lvd);
//fclose(teststrip);
    fprintf(LOGd, "\nDone Getting Light Values");
    fflush(LOGd);
    return;
}

/*--------------------------------------------------------------------------*/
//turns an iteger value into a byte value suitable for pnm
inline unsigned char bytesize(int value)
{
    unsigned char comp;

    comp = value;
    if (value < 0)
	comp = 0;
    if (value > 255)
	comp = 255;
    return (comp);
}

/*--------------------------------------------------------------------------*/
//gets the led times for getting sensor sensitivity information

inline int get_calib_times(unsigned short color, int redmax, int greenmax,
			   int bluemax, int maxbright)
{

    int good = 0;
    unsigned short fixedtime;

    fprintf(LOGd, "\nredmax=%d, greenmax=%d, bluemax=%d", redmax, greenmax,
	    bluemax);
    good = 0;
    fixedtime = fixmaxtime(maxbright, redmax, redtime);
    if (redtime == fixedtime)
	good++;
    else
	redtime = fixedtime;
    if (color == COLOR) {
	fixedtime = fixmaxtime(maxbright, greenmax, greentime);
	if (greentime == fixedtime)
	    good++;
	else
	    greentime = fixedtime;

	fixedtime = fixmaxtime(maxbright, bluemax, bluetime);
	if (bluetime == fixedtime)
	    good++;
	else
	    bluetime = fixedtime;
    } else {
	bluetime = 0.6 * redtime;
	greentime = 0.6 * redtime;

	good += 2;
    }
    fprintf(LOGd, "\nfixed-- red=%d, green=%d, blue=%d", redtime,
	    greentime, bluetime);

    if (good == 3) {
	fprintf(LOGd,
		"\nfound levels, good = 3; redtime=%d, greentime=%d, bluetime=%d",
		redtime, greentime, bluetime);
    }
    return (good);
}

/*--------------------------------------------------------------------------*/
//gets higher led times for good image quality results

inline int get_scan_times(unsigned short color, int redmin, int greenmin,
			  int bluemin, int minbright)
{

    int good = 0;
    unsigned short fixedtime;

    fprintf(LOGd, "\nredmin=%d, greenmin=%d, bluemin=%d", redmin, greenmin,
	    bluemin);
    good = 0;
    fixedtime = fixmintime(minbright, redmin, redtime);
    if (redtime == fixedtime)
	good++;
    else
	redtime = fixedtime;
    if (color == COLOR) {
	fixedtime = fixmintime(minbright, greenmin, greentime);
	if (greentime == fixedtime)
	    good++;
	else
	    greentime = fixedtime;

	fixedtime = fixmintime(minbright, bluemin, bluetime);
	if (bluetime == fixedtime)
	    good++;
	else
	    bluetime = fixedtime;
    } else {
	bluetime = 0.6 * redtime;
	greentime = 0.6 * redtime;
	good += 2;
    }
    fprintf(LOGd, "\nminfixed-- red=%d, green=%d, blue=%d", redtime,
	    greentime, bluetime);

    if (good == 3) {
	fprintf(LOGd,
		"\nfound levels, good = 3; redtime=%d, greentime=%d, bluetime=%d",
		redtime, greentime, bluetime);
    }
    return (good);
}

/*--------------------------------------------------------------------------*/
//calculates the image quality adjustments and stores them in a pseudo
//2 dimensional array  brightness, decontrast, etc. should be made global
//variables, so they can be adjusted as options by interface functions

void make_gamma_table(unsigned char **gamma_table, unsigned short length,
		      unsigned char color)
{

    int value, row, comp, lowboost = 0;
    double diff;
    unsigned char byteval;

    for (row = 0; row < length; row++) {	//for each pixel, each possible input
	for (value = 0; value < 256; value++) {	//value (0 -255) is provided
	    //with a translation
	    comp = lowboost + (value - red_dark[row]) * red_correct[row];
	    diff = 0.95 * comp;
	    if (diff < 0)
		diff = 0;
	    diff = redadjust * brightness
			* (diff + (decontrast * ((255 - diff) * (255 - diff) / 255)));
	    comp = diff;
	    byteval = bytesize(comp);
	    if (color != COLOR)
		gamma_table[row][value] = byteval;

	    if (color == COLOR) {
		gamma_table[row * 3][value] = byteval;	//put red in the right
		//place place
		comp = lowboost + (value - green_dark[row])
		    		* green_correct[row];
		diff = 0.95 * comp;
		if (diff < 0)
		    diff = 0;
		diff = greenadjust * brightness
				* (diff + (decontrast * ((255 - diff) * (255 - diff) / 255)));
		comp = diff;
		byteval = bytesize(comp);
		gamma_table[(row * 3) + 1][value] = byteval;

		comp = lowboost + (value - blue_dark[row]) * blue_correct[row];
		diff = 0.95 * comp;
		if (diff < 0)
		    diff = 0;
		diff = blueadjust * brightness
				* (diff + (decontrast * ((255 - diff) * (255 - diff) / 255)));
		comp = diff;
		byteval = bytesize(comp);
		gamma_table[(row * 3) + 2][value] = byteval;
	    }			//color
	}			//values
    }				//rows
}

/*---------------------------------------------------------------------------*/
//master calibration routine gets all adjustment values needed for a scan

unsigned char **calibrate(int port, unsigned char resolution,
			  unsigned char color)
{
//    FILE *cord;
    int redmin, redmax, greenmin, greenmax, bluemin, bluemax;
    int good = 0, checkbuf, maxbright = 254, minbright = 186;
    unsigned short length = 2550, bytes_per_line;
    unsigned char redbuf[2550], greenbuf[2550], bluebuf[2550];
    int comp, dpi = 0, bytecounter, needed = 1, gettimes = 0;
    unsigned short red_calib_time = 0, green_calib_time =
	0, blue_calib_time = 0;
    unsigned short red_scan_time = 0, green_scan_time = 0, blue_scan_time = 0;
    double darkval, lightval;
    unsigned char **gamma_table;

    fprintf(LOGd, "\nin calibrate; redtime= %d", redtime);
//cord = fopen("corrvals","w+");
    redtime = 0;
    greentime = 0;
    bluetime = 0;

    if (resolution == DPI300)
	dpi = 300;
    if (resolution == DPI200)
	dpi = 200;
    if (resolution == DPI100)
	dpi = 100;
    if (resolution == DPI50)
	dpi = 50;

    bytes_per_line = length = (length * dpi) / 300;
    if (color == COLOR) {
	needed = 3;
	bytes_per_line = 3 * bytes_per_line;
    }
    //if(color == LINEART) length = (length + 7)/8;

    if (uimode != SANEUI)
	printf
	    ("\n           Calibrating...  This will take a minute or so...\n");
    redtime = greentime = bluetime = 0;
    write_register(port, primary_register, MODE);

    get_dark_values(port, resolution, color, 0, 2550);

    checkbuf = 0;
    fprintf(LOGd, "\nthrough the preliminaries");
    /* let's get the data */
//      initialize_calibrate(port,color,resolution,redtime,greentime,bluetime);
    write_register(port, configure_setting_register, 0x10);	//motor off
    write_register(port, line_time_register, 8191);
    write_register(port, empty_image_line_register, 0);
    write_register(port, motor_steps_register, 1000);
    write_register(port, max_lines_register, needed);
    write_register(port, bytes_per_line_register, bytes_per_line);
    usleep(30);

//Get the led time settings
    redtime = 850;		//approximate settings are used to speed calibration
    greentime = 430;
    bluetime = 480;

    while (good < 3) {
	fflush(LOGd);
	write_register(port, red_led_on_time, redtime);
	write_register(port, green_led_on_time, greentime);
	write_register(port, blue_led_on_time, bluetime);

	write_register(port, primary_register, (ENABLE_SCAN | MODE));

	while (checkbuf < needed) {
	    if (MODE == EPP_MODE)
		usleep(30);
	    checkbuf = read_register(port, buffer_line_count_register);

	    if (checkbuf == needed) {
		readbuf(port, redbuf, length);
//                              if(MODE == EPP_MODE) usleep(5);
		if (color == COLOR) {
		    readbuf(port, greenbuf, length);
		    readbuf(port, bluebuf, length);
		}
	    }
	}
	checkbuf = 0;
	write_register(port, primary_register, MODE);
	if ((dpi < 100) && (color != COLOR) && (MODE != NIBBLE_MODE))
	    usleep(5);
//ok , we have the data;  what can we do with it

#define MINMAX(buf,min,max){if (buf < min) min = buf; \
else if (buf > max) max = buf;}

	redmin = greenmin = bluemin = 255;
	redmax = greenmax = bluemax = 0;
	for (bytecounter = 0; bytecounter < length; bytecounter++) {
	    comp = redbuf[bytecounter];
	    MINMAX(comp, redmin, redmax);
	    if (color == color) {
		comp = greenbuf[bytecounter];
		MINMAX(comp, greenmin, greenmax);
		comp = bluebuf[bytecounter];
		MINMAX(comp, bluemin, bluemax);
	    }			//color
	}			//done getting mins and maxs for the line

	if (gettimes == 0) {	//we're getting the calibration levels

	    //get led valuse for the highest sensor val just under maximum.
	    good = get_calib_times(color, redmax, greenmax, bluemax, maxbright);
	    if (redtime > 5000) {
		fprintf(stderr, "\nCalibration times timed out");
		exit(-1);
	    }
	    if (good == 3) {	//we've got the values suitable for calibration
		//now we can move on to get higher values for the actual scan
		red_calib_time = redtime;
		green_calib_time = greentime;
		blue_calib_time = bluetime;
		gettimes = 1;
		good = 0;
		if (((greenmin > minbright) && (redmin > minbright)	//fix for
		     && (bluemin > minbright)) || (redtime > 2000)) {	//non-standard CISs
		    red_calib_time = 600;
		    green_calib_time = 400;	//set default values
		    blue_calib_time = 400;
		    red_scan_time = 800;
		    green_scan_time = 535;
		    blue_scan_time = 535;
		    fprintf(stderr, "setting default led times %d %d %d\n",
			    redmin, greenmin, bluemin);
		    break;
		} else {
		    redtime = redtime * 1.2;	//approximate values for speed
		    greentime = greentime * 1.2;
		    bluetime = bluetime * 1.2;
		}
	    }
	} else {		//we're geting the scan times
	    good =
		get_scan_times(color, redmin, greenmin, bluemin,
			       minbright);
	    if (redtime > 5000) {
		fprintf(stderr, "\nScan times timed out");
		exit(-1);
	    }

	    if (good == 3) {
		red_scan_time = redtime;
		green_scan_time = greentime;
		blue_scan_time = bluetime;
		break;
	    }
	}
    }				//done getting led times

    write_register(port, primary_register, MODE);

    redtime = red_calib_time;	//reset values for calibration
    greentime = green_calib_time;
    bluetime = blue_calib_time;

    get_light_values(port, resolution, color, 0, 2550, redbuf, greenbuf,
		     bluebuf);

    for (bytecounter = 0; bytecounter < 2550; bytecounter++) {
	darkval = red_dark[bytecounter];
	lightval = redbuf[bytecounter];
	red_correct[bytecounter] = (255.0 / (lightval - darkval));	//get full
	//contrast adjuster
//fprintf(cord,"\nbyte %d red %f",bytecounter,red_correct[bytecounter]);

	if (color == COLOR) {
	    darkval = green_dark[bytecounter];
	    lightval = greenbuf[bytecounter];
	    green_correct[bytecounter] = (255.0 / (lightval - darkval));
//fprintf(cord,"\nbyte %d green %f",bytecounter,green_correct[bytecounter]);

	    darkval = blue_dark[bytecounter];
	    lightval = bluebuf[bytecounter];
	    blue_correct[bytecounter] = (255.0 / (lightval - darkval));
//fprintf(cord,"\nbyte %d blue %f",bytecounter,blue_correct[bytecounter]);
	}
    }

    if (color != COLOR)
	gamma_table = init_data_array((2550 * dpi / 300), 256);
    else
	gamma_table = init_data_array((3 * (2550 * dpi / 300)), 256);

    make_gamma_table(gamma_table, (2550 * dpi / 300), color);


//fclose(cord);
    wait_home(port);

    redtime = red_scan_time * led_bright_adjust;	//reset values for scanning
    greentime = green_scan_time * led_bright_adjust;
    bluetime = blue_scan_time * led_bright_adjust;

    fprintf(LOGd, "\nCorrection values calculated");
    if (uimode != SANEUI)
	printf("\nBeginning scan.");
    return gamma_table;
}
