/***************************************************************************
                          cmdline.c  -  description
                             -------------------
    begin                : Sat Feb 12 2000
    copyright            : (C) 2000 by Eugene Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"



/*--------------------------------------------------------------------------*/
//This function makes a single scan using parameters taken from the -c option.

void cmd_line(int port, int argc, char **argv)
{
    unsigned char color, resolution;
    unsigned short startpos, stoppos, startline, stopline;
    float leftpos, rightpos, frontline, backline;
    int divisor = 1;

    switch (atoi(argv[1])) {
    case (50):
	resolution = DPI50;
	break;
    case (100):
	resolution = DPI100;
	break;
    case (200):
	resolution = DPI200;
	break;
    case (300):
	resolution = DPI300;
	break;
    default:
	printf("\nSorry, %s is not a valid resolution.", argv[1]);
	printf("\nValid modes are 50,100,200, and 300.\n");
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }
    ++argv;

    if ((strcasecmp(argv[1], "color") == 0)
	|| (strcasecmp(argv[1], "colour") == 0))
	color = COLOR;
    else if ((strcasecmp(argv[1], "grey") == 0)
	     || (strcasecmp(argv[1], "gray") == 0)
	     || (strcasecmp(argv[1], "greyscale") == 0)
	     || (strcasecmp(argv[1], "grayscale") == 0)) {
	color = GREYSCALE;
	lineart = 0;
    } else if ((strcasecmp(argv[1], "lineart") == 0)) {
	color = GREYSCALE;
	lineart = 1;
    } else {
	printf("\nSorry, %s is not a valid color mode.", argv[1]);
	printf("\n Valid modes are color, grey, and lineart.\n");
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }
    ++argv;

    if ((atof(argv[1]) >= 0) && (atof(argv[1]) <= 8.5)) {
	leftpos = atof(argv[1]);
	startpos = 300 * leftpos;
    } else {
	printf
	    ("\nSorry, %s is not a valid position.\nIt must be between 0 and 8.5.\n",
	     argv[1]);
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }
    ++argv;

    if ((atof(argv[1]) >= leftpos) && (atof(argv[1]) <= 8.5)) {
	rightpos = atof(argv[1]);
	stoppos = 300 * rightpos;
    } else {
	printf
	    ("\nSorry, %s is not a valid position.\nIt must be between %f and 8.5.\n",
	     argv[1], leftpos);
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }
    ++argv;

    if ((atof(argv[1]) >= 0) && (atof(argv[1]) <= 11.7)) {
	frontline = atof(argv[1]);
	startline = 300 * frontline;
    } else {
	printf
	    ("\nSorry, %s is not a valid position.\nIt must be between 0 and 11.7.\n",
	     argv[1]);
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }
    ++argv;

    if ((atof(argv[1]) >= frontline) && (atof(argv[1]) <= 11.7)) {
	backline = atof(argv[1]);
	stopline = 300 * backline;
    } else {
	printf
	    ("\nSorry, %s is not a valid position.\nIt must be between %f and 11.7.\n",
	     argv[1], frontline);
	printf
	    ("\n\nFormat for command line mode is -c resolution {50 - 300}");
	printf
	    ("\ncolor {color, grey, lineart} left edge of scan {0 - 8.5}");
	printf
	    ("\nin inches - 1 inch = 25.4mm}  right edge of scan {left - 8.5}");
	printf
	    ("\nfront edge of scan {0 - 11.7} back edge {front - 11.7}\n");
	return;
    }

    if ((resolution == DPI200) || (resolution == DPI100))
	divisor = 3;
    else if (resolution == DPI50)
	divisor = 6;		//get legal values for 200 dpi
    startpos = (startpos / divisor) * divisor;
    stoppos = (stoppos / divisor) * divisor;
    startline = (startline / divisor) * divisor;
    stopline = (stopline / divisor) * divisor;

    scan(port, resolution, color, startpos, stoppos, startline, stopline);
}
