/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Tue Dec 28 21:15:10 EST 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"


/*--------------------------------------------------------------------------*/
//figures out in which parallel port modes the driver can communicate
//successfully with the scaner

unsigned char getmodes(port)
{
    unsigned short test;
    unsigned char modes = 0;

    set_mode(port, ECR_NIBBLE, NIBBLE_MODE);
    write_register(port, green_led_on_time, 0x4252);
    test = read_register(port, green_led_on_time);
//      printf("\ntest = 0x%x",test);
    if (test == 0x4252)
	modes |= 0x1;

    set_mode(port, ECR_BYTE, BYTE_MODE);
    write_register(port, green_led_on_time, 0x5242);
    test = read_register(port, green_led_on_time);
//      printf("\ntest = 0x%x",test);
    if (test == 0x5242)
	modes |= 0x2;

    set_mode(port, ECR_EPP, EPP_MODE);
    write_register(port, green_led_on_time, 0x6272);
    test = read_register(port, green_led_on_time);
//      printf("\ntest = 0x%x",test);
    if (test == 0x6272)
	modes |= 0x4;

    return (modes);
}

/*--------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
    int port, portaccess;
    unsigned char modes, oldportmode = 0, setmode =
	0, oldctl, oldstat, olddata;

    uimode = 0;

    while ((argc > 1) && (argv[1][0] == '-')) {
	switch (argv[1][1]) {
	case 'm':
	    if ((strcasecmp(argv[2], "nibble") == 0)
		|| (strcasecmp(argv[2], "spp") == 0))
		setmode = 1;	//to set to nibble mode
	    else if ((strcasecmp(argv[2], "byte") == 0)
		     || (strcasecmp(argv[2], "ps/2") == 0)
		     || (strcasecmp(argv[2], "ps2") == 0)
		     || (strcasecmp(argv[2], "bidir") == 0))
		setmode = 2;	//to set to byte mode
	    else if ((strcasecmp(argv[2], "epp") == 0))
		setmode = 3;	//to set to epp mode
	    else {
		fprintf(stderr, "\nSorry, %s is not a valid port mode.\n",
			argv[2]);
		fprintf(stderr, "\nTry \"nibble\", \"byte\", or \"epp\".");
		uimode = -1;
	    }
	    ++argv;
	    --argc;
	    break;

	case 'c':
	    uimode = 1;		//command line mode
	    break;

	case 't':
	    uimode = 2;		//test mode
	    break;

	case 's':
	    uimode = 3;		//SANE ui
	    sscanf(argv[2], "%d", &ctloutpipe);
	    sscanf(argv[3], "%d", &ctlinpipe);
	    sscanf(argv[4], "%d", &datapipe);
	    break;

	default:
	    printf("\nSorry, %s is not a valid option.\n", argv[1]);
	    printf
		("\nas6edriver version %s\nhttp://as6edriver.sourceforge.net\n",
		 VERSION);
	    printf
		("\nOptions:\n\n-c   Use command line mode. {-c dpi color left right front back}");
	    printf
		("\n-m   Set port mode, {nibble,byte,epp} overriding automatic detection.");
	    printf("\n-t   Run tests.\n");
	    uimode = -1;
	}
	++argv;
	--argc;
    }

    port = get_port_address();
    portaccess = iopl(3);
    if (portaccess != 0) {
	fprintf(stderr, "\n       Port access denied.  Bummer....\n");
	if (uimode == SANEUI) {
	    uimode = -2;
	    write(ctloutpipe, &uimode, sizeof(uimode));
	}
	return (-1);
    }

    initialize_regdata();
    LOGd = fopen("/var/tmp/as6e.log", "w+");

    oldportmode = inb(port + 0x402);	//save original port status
    oldctl = inb(controlport(port));
    oldstat = inb(statusport(port));
    olddata = inb(port);

    outb((0x30 | (oldportmode & 0x1f)), port + 0x402);	//Fixes Winbond ecr lock
    outb((0x10 | (oldportmode & 0x1f)), port + 0x402);	//Fixes prob caused by
    // fixing Winbond ecr lock
    wake_up_scanner(port);
    set_mode(port, ECR_NIBBLE, NIBBLE_MODE);

    modes = getmodes(port);

    if ((modes & 0x4) != 0) {
	set_mode(port, ECR_EPP, EPP_MODE);
//              printf("\nUsing EPP Mode.");
	fprintf(LOGd, "\nEPP mode");
    } else if ((modes & 0x2) != 0) {
	set_mode(port, ECR_BYTE, BYTE_MODE);
//              printf("\nUsing Byte Mode.");
	fprintf(LOGd, "\nbyte mode");
    } else if ((modes & 0x1) != 0) {
	set_mode(port, ECR_NIBBLE, NIBBLE_MODE);
	fprintf(stderr, "\nUsing Nibble Mode.");
	fprintf(stderr, "\nScanning will be very slow in this mode,");
	fprintf(stderr, "\nand scan quality will be inferior.");
	fprintf(stderr,
		"\nConsider changing the parallel port mode in your BIOS");
	fprintf(stderr, "\nor see http://as6edriver.sourceforge.net");
	fprintf(LOGd, "\nnibble mode");
	usleep(3);
    } else {
	if (uimode == SANEUI) {
	    uimode = -1;
	    write(ctloutpipe, &uimode, sizeof(uimode));
	} else {
	    printf("\nUnable to contact the scanner.  Exiting.\n");
	    uimode = -1;
	}
    }

    if (setmode) {
	switch (setmode) {
	case (1):
	    set_mode(port, ECR_NIBBLE, NIBBLE_MODE);
	    printf("\nResetting to nibble mode.\n");
	    break;
	case (2):
	    set_mode(port, ECR_BYTE, BYTE_MODE);
	    printf("\nResetting to byte mode.\n");
	    break;
	case (3):
	    set_mode(port, ECR_EPP, EPP_MODE);
	    printf("\nResetting to EPP mode.\n");
	    break;
	}
    }

    switch (uimode) {
    case (0):
	console(port);		//enter interactive text based interface
	break;
    case (1):
	cmd_line(port, argc, argv);
	break;
    case (2):
	testsuite(port);
	break;

    case (3):
	sane(port);
	break;

    default:
	break;
    }

    if (uimode != SANEUI)
	printf("\nReleasing parallel port.\n");
    write_register(port, primary_register, 0x40);
    fclose(LOGd);
    outb(oldportmode, (port + 0x402));	//restore original port status
    outb(oldctl, controlport(port));
    outb(oldstat, statusport(port));
    outb(olddata, port);
    return EXIT_SUCCESS;
}
