/***************************************************************************
                          port.c  -  description
                             -------------------
    begin                : Fri Mar 24 2000
    copyright            : (C) 2000 by Eugene Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
//This file contains the routines for parsing the config file.

#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"


//Writes /etc/as6e.conf if none is found.

void write_conf(void)
{
    FILE *confd;

    confd = fopen("/etc/as6e.conf", "w+");
    if (confd != NULL) {
	fprintf(confd,
		"#This is the configuration file for the as6edriver.");
	fprintf(confd,
		"\n#Please uncomment the address for the parallel port");
	fprintf(confd, "\n#to which the scanner is attached.\n");
	fprintf(confd,
		"\n#If you have the /proc file system (most recent ");
	fprintf(confd,
		"\n#distributions do) use /dev/parportX, otherwise use");
	fprintf(confd, "\n#the numerical address.\n\n");
	fprintf(confd, "\n/dev/parport0");
	fprintf(confd, "\n#0x378");
	fprintf(confd, "\n#/dev/parport1");
	fprintf(confd, "\n#0x278");
	fprintf(confd, "\n#/dev/parport2");
	fprintf(confd, "\n#0x3bc");
	fclose(confd);
    } else
	fprintf(stderr, "\nUnable to create the /etc/as6e.conf file.");
}

//This function gets the port address from the config file, or by looking
//it up in the /proc/parport directory. 

int get_port_address(void)
{

    struct stat statbuf;
    mode_t statmodes;
    int outcome, portctr, strcounter, offset = 0, base, port, done =
	0, foundport = 0;
    long longport;
    FILE *procfd, *confd;
    char procinfo[128], portstr[8], devstr[128], procloc[64],
	portnumber[2], *gotstr;

    confd = fopen("/etc/as6e.conf", "r");
    if (!confd) {
	fprintf(stderr, "\nNo configuration file found:");
	fprintf(stderr,
		"\nDefaulting to /dev/parport0 and writing /etc/as6e.conf");
	fprintf(stderr,
		"\nPlease edit the file if /dev/parport0 is not correct.");
	strcpy(procloc, "/proc/parport/0");
	write_conf();
    } else {
	while (!done) {
	    gotstr = fgets(devstr, sizeof(devstr), confd);
	    if (gotstr != NULL) {
		switch (devstr[0]) {
		case '#':
		    break;
		case '\n':
		    break;
		case '0':
		    if (sscanf(devstr, "%x", &port))
			return port;
		    break;
		case '/':
		    if (strncmp(devstr, "/dev/parport", 12) == 0) {
			portnumber[0] = devstr[12];
			portnumber[1] = '\0';
			strcpy(procloc, "/proc/parport/");
			strcat(procloc, portnumber);
			done = 1;
		    } else
			fprintf(stderr, "\n%s is not a valid port address",
				devstr);
		    break;
		default:
		    fprintf(stderr, "\n%s is not a valid port address",
			    devstr);
		    break;
		}
	    } else {
		fprintf(stderr,
			"\n/etc/as6e.conf contains no valid port address");
		fprintf(stderr, "\ndefaulting to /dev/parport0");
		strcpy(procloc, "/proc/parport/0");
		done = 1;
	    }
	}
	fclose(confd);
    }
/*  Assume a 2.2.X kernel-- see below for later versions */
    stat(procloc, &statbuf);
    statmodes = statbuf.st_mode;
    outcome = S_ISDIR(statmodes);
    if (outcome) {
	strcat(procloc, "/hardware");
	procfd = fopen(procloc, "r");
	fread(procinfo, 128, 1, procfd);
	fclose(procfd);
	for (strcounter = 0; strcounter < 120; strcounter++) {
	    if (procinfo[strcounter] == 'b') {
		if ((procinfo[strcounter + 1] == 'a')
		    && (procinfo[strcounter + 2] == 's')
		    && (procinfo[strcounter + 3] == 'e')) {
		    for (portctr = 0; portctr < 8; portctr++) {
			base = strcounter + 3 + portctr;
			if (procinfo[base] == '0') {
			    while ((procinfo[base + offset] != ' ')
				   && (procinfo[base + offset] != '\n')
				   && (offset < 8)) {
				portstr[offset] = procinfo[base + offset];
				offset++;
			    }
			    break;
			}
		    }
		    break;
		}
	    }
	}
	sscanf(portstr, "%x", &port);
    }

    else {
	stat("/proc/ioports", &statbuf);
	statmodes = statbuf.st_mode;
	outcome = S_ISREG(statmodes);
	if (outcome); {		//2.4 kernel
	    procfd = fopen("/proc/ioports", "r");
	    while (!foundport) {
		gotstr = fgets(procinfo, sizeof(procinfo), procfd);
		if (gotstr == NULL)
		    break;	//No parport found in the file
		if ((strncmp((procinfo + 12), "parport", 7) == 0)
		    && (procinfo[19] == portnumber[0])) {
		    longport = strtol(procinfo, (char **) NULL, 16);
		    port = longport;
		    fclose(procfd);
		    foundport = 1;
		    return port;
		}
	    }
	}

	switch (portnumber[0]) {
	case '1':
	    port = 0x278;
	    break;
	case '2':
	    port = 0x3bc;
	    break;
	default:
	    port = 0x378;
	    break;
	}
	fprintf(stderr,
		"\nProc directory not found.  Defaulting to port 0x%x.",
		port);
    }
    return port;
}
