/***************************************************************************
                          readbuf.c  -  description
                             -------------------
    begin                : Wed Dec 29 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/***************************************************************************
                          readbuf.c  -  description
                             -------------------
    begin                : Sun Dec 26 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/* These functions read data from the scanner during a scan.  They are
optimized for speed, unlike the register read/write functions, which are
optimized for reliability. */

#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"


/*--------------------------------------------------------------------------*/

//reads a character in nibble mode (from Luigi Rizzo's ppscan.c)

inline unsigned char nibble_char(port)
{
    unsigned char ctl, c, ix, buf[2];

#define nibble2char(s) (0xf & (((s & ~nACK) >> 3) | (~s & nBUSY) >> 4))

    ctl = inb(controlport(port));	//printf("\nctl0= 0x%x",ctl);
    c = (ctl | nINIT) & ~(AUTOFEED | STROBE | SELECTIN);
    if (c != ctl)
	fprintf(stderr, "\nnibble status bad: 0x%02x ought to be 0x%02x",
		ctl, c);

    for (ix = 0; ix < 2; ix++) {
	c |= AUTOFEED;		//host ready for nibble
	if (ix == 0)
	    c &= ~nINIT;	//tell scanner to start nibbling out
	outb(c, controlport(port));
	buf[ix] = inb(statusport(port));	//  printf("\nnibble = 0x%x",buf[ix]);
	c &= ~AUTOFEED;		//got the data
	c |= nINIT;		//keep going
	outb(c, controlport(port));
	buf[ix] = nibble2char(buf[ix]);
    }
    buf[0] = buf[0] | (buf[1] << 4);

    return (buf[0]);
}

/*--------------------------------------------------------------------------*/

//reads a character in byte mode

inline unsigned char byte_char(port)
{
    unsigned char ctl, oldctl, data;

    ctl = inb(controlport(port));
    oldctl = ctl;
    ctl |= (DIRECTION | AUTOFEED);
    outb(ctl, controlport(port));
    data = inb(port);
    outb(oldctl, controlport(port));

    return (data);

}

/*--------------------------------------------------------------------------*/
//reads a line of characters

inline unsigned short readbuf(int port, unsigned char *buffer, int length)
{

    int counter;
    unsigned char addr, ctl;

    addr = REGISTER[sram_test_register].low;

    if (MODE == EPP_MODE) {
	clear_epp_timeout(port);
	outb(addr, eppaddressport(port));
    } else {
	outb_p(addr, port);
	ctl = inb_p(controlport(port));
	ctl |= STROBE;
	outb_p(ctl, controlport(port));
	ctl |= SELECTIN;
	outb_p(ctl, controlport(port));
	ctl &= ~SELECTIN;
	outb_p(ctl, controlport(port));
	ctl &= ~STROBE;
	outb_p(ctl, controlport(port));
    }

    for (counter = 0; counter < length; counter++) {

	if (MODE == EPP_MODE)
	    buffer[counter] = inb(eppdataport(port));
	else if (MODE == NIBBLE_MODE)
	    buffer[counter] = nibble_char(port);
	else if (MODE == BYTE_MODE)
	    buffer[counter] = byte_char(port);
    }
    if (MODE == EPP_MODE)
	clear_epp_timeout(port);
    return (length);
}
