/***************************************************************************
                          regdata.h  -  description
                             -------------------
    begin                : Fri Dec 24 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
/*	This header contains the info necessary for using the scaner's registers*/



#define BYTE	0
#define WORD 	1




struct REGDATA {		//Prototype for scanner register info
    unsigned char low;		//register address
    unsigned char high;		//only for WORD types
    int type;			//BYTE or WORD
};

typedef struct REGDATA REGTYPE;

REGTYPE REGISTER[29];


#define primary_register 0	//this is the primary control register: r/w
#define RESET_SCANNER					0x00
#define	RESTORE_PASSTHROUGH   			0x40
#define ENABLE_SCAN						0x80
#define NIBBLE_MODE						0x00
#define BYTE_MODE						0x10
#define	EPP_MODE						0x20
#define ECP_MODE							0x30

#define status_control_register 1	//r only: reads the scanner's current state
#define PAPER_IN(status)				((status & 1<<7) == 1<<7)
#define PAPER_IN_POSITION(status)		((status & 1<<6) == 1<<6)
#define MOTOR_AT_END_POSITION(status) 	((status & 1<<5) == 1<<5)
#define AUTOSCAN_BUTTON_PUSHED(status)	((status & 1<<4)== 1<<4)
#define SENSORS_CHOSEN(status) \
((2*((~status & 1<<3) == 1<<3)+((~status & 1<<2) == 1<<2))
#define FLATBED_SCANNER(status)			(status & 1<<0)
#define SHEET_FED_SCANNER(status) 		(~status & 1<<0)


#define configure_setting_register 2	// r/w: contols some basic settings
#define ACKNOWLEDGE_AUTOSCAN 		0x80
#define TEST_SRAM					0x40
#define ENABLE_SW_MOTOR_CTL			0x20
#define TURN_OFF_MOTOR				0x10
#define MOTOR_BACKWARD				0x08
#define ENABLE_WATCHDOG_TIMER		0x04
#define CISCLK_2						0x01


#define scan_mode_register 3	// r/w: sets scan mode
#define DPI50								0x08	//pixel average on for 50dpi
#define DPI100							0x40
#define DPI200							0x80
#define DPI300							0xc0
#define HWLINEART		   					0x10
#define COLOR							0x20
#define GREYSCALE							0x30
#define PIXEL_AVERAGE_MODE 				0x08
#define LINEART_ENHANCE 					0x04
#define VERTICAL600		 				0x02



#define	line_time_register   4	// r/w: how many pixel clocks per scan line
//Unit = 8*system clk  16 bit


#define image_pixel_start 5	// r/w: how far from the left to begin
//in pixels!, 12 bit


#define image_pixel_stop 6	//  r/w: how far from the right to stop
//in pixels!, 12 bit


#define lineart_threshold_up  7	// r/w: Threshold for line art mode:
//I don't really understand thisif I have time, I'll hack on this to see
//how it behaves .


#define lineart_threshold_down  8	// r/w: Threshold for line art mode down:
//I don't really understand this if I have time, I'll hack on this to see
//how it behaves.


//Sets how many steps the motor should continue
// w/o receiving any data. Units= steps, 12 bit, but (0x0c,0x0b) = 64 ksteps
//  r/w
#define empty_image_line_register 9



//Sets how many steps the motor should go
// when under software control.
// Reading will determine the no.of steps left before a sensor-stop is reached
//12 bits, Units = steps, but (0x0c,0x0b) = 64k steps.
#define motor_steps_register 10



#define motor_rotation_speed 11	//period = 1 step, units = system clk
#define MOTOR_SPEED_SLOW = 0x50	//min?
#define MOTOR_SPEED_FAST = 0xc3	//max?



#define led_control 12
#define LED_SW_CONTROL 				0x80
#define RED_LED_ON					0x40
#define GREEN_LED_ON					0x20
#define BLUE_LED_ON					0x10


#define red_led_on_time 13	//red LED on time for 1 scan line.
//Unit = 8* sytem clk  16bit



#define green_led_on_time 14	//green LED on time for 1 scan line.
//doc. says Unit = Pixels Cl, but is probably the same as for the red and
//blue LEDs.


#define blue_led_on_time 15	//blue LED on time for 1 scan line.
//Unit = 8* sytem clk  16bit


#define max_lines_register 16	//sets the maximum nuber of lines to store in
// SRAM before stoping the scan.  Unit=lines 8 bit


#define bytes_per_line_register 17	//How many bytes stored in SRAM for one line
//Oneline=actual value - 1  Unit =  bytes  12 bits
//8 bits


#define sram_test_register 18	// lets PC write to the image buffer for testing

#define buffer_addr_test_register 19	//tells to what address SRAM is pointed
//16 bits


#define buffer_line_count_register 20	//number of lines in the image buffer
//8 bits


#define motor_hold_time_register 21	//In doc is says this is how long the motor
//will be moving.  The name would indicate the lengthof a pause.
//Unit =1.67ms        Maximum time = 427.8ms  8 bit


#define motor_back_step_register 22	//How many steps the motor will move back
//Unit = 16 steps        Maximum = 4080 steps, 8 bit


#define darkhead_register  23	//How many pixels will be ignored before the
//led is turned on  8 bit  r/w


//I think that the "motor" in the next two refers to the sheet-feeder motor.
//I have no way of testing this hpothesis.

#define motor_head_register 24	//in sheet fed mode, will set how many steps to
//move until paper in and paper in line = 1. Unit = 16 steps default = 4080 
//8 bit


#define motor_tail_register 25	//in sheet fed mode, will set how many steps to
//move when scanning is through.  Unit = 16 steps, default = 4080  8 bit


#define watchdog_timer_register 26	//sets how long the scanner is to stop its
//walk after the PC doesn't read or write to it  Unit = 1.6s Maximum = 408s
//8 bit


#define paper_jam_register 27	// r only for sheet-fed scanners
#define PAPER_JAM					0x80	//Most of the doc for this is
#define	RESET_PAPER_JAM_SIGNAL 	0x08	//impenatrable to me.
