/***************************************************************************
                          sane.c  -  description
                             -------------------
    begin                : Fri Feb 18 2000
    copyright            : (C) 2000 by Eugene Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"

/*--------------------------------------------------------------------------*/
double adjust_brightness(double val)
{
    double bright;
    if (val >= 0) {
	bright = 1.0 + (val / 100.0);
	led_bright_adjust = 1.0;
    } else {
	bright = 1.0;
	led_bright_adjust = (100.0 + val) / 100.0;
    }
    return bright;
}

/*--------------------------------------------------------------------------*/

void sane(int port)
{

    int scan_params[8], as6estatus, colormode, dpi, repeat =
	1, scan_complete = -1;
    unsigned short color, resolution;
    unsigned short startpos, stoppos, startline, stopline;

    switch (MODE) {
    case (NIBBLE_MODE):
	as6estatus = 1;
    case (BYTE_MODE):
	as6estatus = 2;
    case (EPP_MODE):
	as6estatus = 3;
    }
    write(ctloutpipe, &as6estatus, sizeof(as6estatus));
//              read(ctlinpipe,&repeat,sizeof(repeat));                 
    read(ctlinpipe, &repeat, sizeof(repeat));
//printf("\nbefore read repeat = %d",repeat); fflush(stdout);

    while (repeat) {
	signal(SIGUSR1, cancel);
//printf("\nbefore read params"); fflush(stdout);
	read(ctlinpipe, scan_params, sizeof(scan_params));

	dpi = scan_params[0];
	colormode = scan_params[1];
	startpos = scan_params[2];
	stoppos = scan_params[3];
	startline = scan_params[4];
	stopline = scan_params[5];
	brightness = adjust_brightness(scan_params[6]);
	decontrast = scan_params[7] / -200.0;

//      for(i = 0; i < 8 ; i++) printf("%d ",scan_params[i]);
//fflush(stdout);

	switch (dpi) {
	case (50):
	    resolution = DPI50;
	    break;
	case (100):
	    resolution = DPI100;
	    break;
	case (200):
	    resolution = DPI200;
	    break;
	case (300):
	    resolution = DPI300;
	    break;
	default:
	    fprintf(stderr, "\nInvalid resolution");
	    as6estatus = 0;
	    write(ctloutpipe, &as6estatus, sizeof(as6estatus));
	    return;
	}
	switch (colormode) {
	case (0):
	    color = COLOR;
	    break;
	case (1):
	    color = GREYSCALE;
	    lineart = 0;
	    break;
	case (2):
	    color = GREYSCALE;
	    lineart = 1;
	    break;
	default:
	    fprintf(stderr, "\nInvalid color mode");
	    as6estatus = 0;
	    write(ctloutpipe, &as6estatus, sizeof(as6estatus));
	    return;
	}
	if ((startpos > 2550) || (stoppos < startpos) || (stoppos > 2550)
	    || (startline > 3510) || (stopline < startline)
	    || (stoppos > 3510)) {

	    fprintf(stderr, "\nInvalid geometry");
	    as6estatus = 0;
	    write(ctloutpipe, &as6estatus, sizeof(as6estatus));
	    return;
	}

	scan(port, resolution, color, startpos, stoppos, startline,
	     stopline);
	write(ctloutpipe, &scan_complete, sizeof(scan_complete));
//printf("\nWrote       -1 for eof"); fflush(stdout);
	read(ctlinpipe, &repeat, sizeof(repeat));
//printf("\nafter read repeat = %d",repeat); fflush(stdout);
	if (repeat)
	    wait_home(port);
    }
    close(datapipe);
    close(ctloutpipe);

}
