/***************************************************************************
                          scan.c  -  description
                             -------------------
    begin                : Sun Dec 26 1999
    copyright            : (C) 1999 by Eugene S. Weiss
    email                : eweiss@sas.upenn.edu
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*  This file contains the master scan routine.  																								*/

#include "as6edriver.h"
#include "regdata.h"
#include "prototypes.h"

/*--------------------------------------------------------------------------*/

void free_data_array(unsigned char **array, unsigned short rows)
{
    unsigned short rowcounter;

    for (rowcounter = 0; rowcounter < rows; rowcounter++)
	free(array[rowcounter]);
    free(array);
}

/*--------------------------------------------------------------------------*/
//orders the scan routine to abort

void cancel(int sig)
{
    fprintf(stderr, "\nreceived cancel signal");
    cancelled = 1;
}

/*--------------------------------------------------------------------------*/
//checks that scanner hasn't crashed or been turned off
int check_for_scanner(int port)
{
    int test;
    write_register(port, green_led_on_time, 0x42);
    test = read_register(port, green_led_on_time);
    if (test != 0x42) {
	fprintf(stderr, "Scanner not responding\n");
	return (0);
    }
    write_register(port, green_led_on_time, 0x00);
//          printf("\n             Scanner detected.\n");
    return (1);
}

/*--------------------------------------------------------------------------*/
//standard function for checking memory availability before dynamic allocation
void *xmalloc(size_t size)
{
    register void *value;

    value = malloc(size);

    if (value == 0) {
	fprintf(stderr, "\nUnable to allocate memory\n");
	fflush(stderr);
	exit(1);
    }
    return value;
}

/*--------------------------------------------------------------------------*/
//this function allocates a 2 dimensional array of unsigned bytes

unsigned char **init_data_array(unsigned short rows,
				unsigned short coloumns)
{
    unsigned short rowcounter;
    unsigned char **array;

    array = (unsigned char **) xmalloc(rows * sizeof(unsigned char *));

    for (rowcounter = 0; rowcounter < rows; rowcounter++)
	array[rowcounter] = (unsigned char *) xmalloc(coloumns);
    return array;
}

/*--------------------------------------------------------------------------*/
void write_pnm_headers(FILE * fd, unsigned char color,
		       unsigned short bytes_per_line, unsigned short lines)
{

    if ((color == GREYSCALE) && lineart)
	fprintf(fd, "P5\n%d %d\n255\n", /*( */
		bytes_per_line /* + 7) / 8 */ , lines);
    if ((color == GREYSCALE) && (lineart == 0))
	fprintf(fd, "P5\n%d %d\n255\n", bytes_per_line, lines);
    if (color == COLOR)
	fprintf(fd, "P6\n%d %d\n255\n", bytes_per_line, lines);
}

/*--------------------------------------------------------------------------*/
//This function turns a greyscale image into a b/w one.

void pack_lineart(unsigned char *greydata, int length)
{
    int bytecounter;
    unsigned char threshold = 0xa0;

    for (bytecounter = 0; bytecounter < length; bytecounter++) {
	if (greydata[bytecounter] > threshold)
	    greydata[bytecounter] = 0xff;
	else
	    greydata[bytecounter] = 0x0;
    }


/*		value = 0;
		if(greydata[(bytecounter * 8) + 0] > threshold) value |= 80;
		if(greydata[(bytecounter * 8) + 1] > threshold) value |= 40;
        if(greydata[(bytecounter * 8) + 2] > threshold) value |= 20;
        if(greydata[(bytecounter * 8) + 3] > threshold) value |= 10;
        if(greydata[(bytecounter * 8) + 4] > threshold) value |= 8;
        if(greydata[(bytecounter * 8) + 5] > threshold) value |= 4;
        if(greydata[(bytecounter * 8) + 6] > threshold) value |= 2;
        if(greydata[(bytecounter * 8) + 7] > threshold) value |= 1;
		lineartarray[bytecounter] = value; //printf(" %d",value); */

}

/*--------------------------------------------------------------------------*/
/*this is the main scan routine. Send it scan dimensions, color and resolution
and it directs the scan process, including calibration in its entirety*/

void scan(int port, unsigned char resolution, unsigned char color,
	  unsigned short startpos, unsigned short stoppos,
	  unsigned short startline, unsigned short stopline)
{

    unsigned char redbuffer[2550], greenbuffer[2550], bluebuffer[2550];
    FILE *fd = NULL;
    unsigned short max_lines, bytes_per_line, lines, readylines, needed,
	neededcounter;
    unsigned short linetime, minlinetime = 3000, motorspeed =
	0xc350, linecounter, setting;
    int test, pause = 0, dpi = 50, offset =
	180, red, green, blue, waitcounter, bytecounter;
    unsigned char **data_array, **gamma_table /* ,lineartarray[320] */ ;
    int writecounter, inbuffer, sleepcounter, written =
	0, writelength, leftalign /*,imagecounter = 0 */ ;
    long fspos;

    fprintf(LOGd, "\nScanning area x- %d to %d, y- %d to %d", startpos,
	    stoppos, startline, stopline);

    cancelled = 0;
    signal(SIGUSR1, cancel);

    if (resolution == DPI300)
	dpi = 300;
    if (resolution == DPI200)
	dpi = 200;
    if (resolution == DPI100)
	dpi = 100;
    if (resolution == DPI50)
	dpi = 50;

    leftalign = (startpos * dpi) / 300;

    test = check_for_scanner(port);	//scanner hasn't gone down
    if (test == 0)
	return;
    //total number of lines (rgb or grey) that will be scanned
    lines = ((stopline * dpi) / 300) - ((startline * dpi) / 300);

    //length of the lines to be written-- needed for the PNM headers:
    //the scanner internally counts 3 lines ofr each color line
    writelength = bytes_per_line =
	((stoppos * dpi / 300) - (startpos * dpi / 300));
    if (color == COLOR) {
	writelength = (3 * bytes_per_line);
	needed = 3;
    }

    if ((color == COLOR) && (dpi == 50))
	minlinetime = 5000;

    //keeping the linetimes down for short lines in slower modes can
    //avoid the necessity of pausing between reading the data and
    //checking the number of lines in the buffer
    linetime = (writelength * 3) / ((MODE / 0x10) + 1);
    if (linetime < minlinetime) {
	linetime = minlinetime;
	pause = 1;
    }
    if (linetime > 8191)
	linetime = 8191;
    if ((color == GREYSCALE) && (dpi < 200)) {
	pause = 1;
	linetime = 8191;
    }
    if ((color != COLOR) && (dpi < 200)) {
	pause = 1;
	linetime = 8191;
    }
    if (dpi == 50)
	linetime = 8191;
    if (MODE == EPP_MODE)
	linetime = 8191;
//fprintf(LOGd,"\nLinetime = %d",linetime); fflush(stdout);
    //this is the most the scanners internal buffer can hold without generating
    //errors
    max_lines = 9 * 2550 / bytes_per_line;
    if (max_lines > 255)
	max_lines = 255;

    if (uimode != SANEUI)
	fd = fopen("as6etemp.pnm", "w+");	//open temporary file-- this should be
    //changed to a global variable  
    fprintf(LOGd, "\nstartline = %d stopline = %d", startline, stopline);

    needed = 3 * ((max_lines * 3) / 12);	//start reading at 3/4 of a full
    //hardware buffer.  Color needs a
    //multiple of 3
//      if((color == GREYSCALE) && (lines < needed)) needed = lines;
//      if((color == COLOR) && ((lines * 3) < needed)) needed = lines * 3;

    //write PNM headers
    if (uimode != SANEUI)
	write_pnm_headers(fd, color, bytes_per_line, lines);

    fprintf(LOGd, "\nlines= %d,bpl=%d, maxl= %d", lines, bytes_per_line,
	    max_lines);
    gamma_table = calibrate(port, resolution, color);

    write_register(port, primary_register, MODE);

    write_register(port, red_led_on_time, redtime);
    write_register(port, green_led_on_time, greentime);
    write_register(port, blue_led_on_time, bluetime);
    red = read_register(port, red_led_on_time);
    green = read_register(port, green_led_on_time);
    blue = read_register(port, blue_led_on_time);
    write_register(port, configure_setting_register, 0x0);
    fprintf(LOGd, "\nRed=0x%x, Green= 0x%x, Blue= 0x%x", red, green, blue);

    write_register(port, line_time_register, linetime);
    write_register(port, motor_rotation_speed, motorspeed);

    write_register(port, image_pixel_start, startpos);	//start at the left edge
    write_register(port, image_pixel_stop, stoppos);	//end at the right edge
    write_register(port, empty_image_line_register, (startline + offset));	//empty steps
    write_register(port, max_lines_register, max_lines);
    write_register(port, motor_steps_register, (stopline + offset));
    write_register(port, bytes_per_line_register, bytes_per_line);

    //make the array of reading the data into.  It's too big for color,
    //but still small enough not to matter.  It'll be fixed when it's
    //replaced by a big image buffer
    data_array = init_data_array(needed, writelength);

    usleep(100000);		//tenth second delay to make sure all the registers
    //have communicated with the scanner's internal controller
    //on fast systems

    //make sure the motor is on before scan begins
    for (waitcounter = 0; waitcounter < 10000; waitcounter++) {
	setting = read_register(port, configure_setting_register);
	if (setting == 0) {
	    write_register(port, primary_register, (ENABLE_SCAN | MODE));	//printf("Scanning 0x%x",(ENABLE_SCAN | MODE));
	    break;
	}
	usleep(500);
	if (waitcounter > 9990) {
	    fprintf(stderr,
		    "\nWe've got a problem, please see http://as6edriver.sourceforge.net");
	    test12(port);
	}
    }
    sleepcounter = 0;
    //start reading the scan
    linecounter = 0;
    while (linecounter < lines) {

	if ((color == GREYSCALE) && ((lines - linecounter) <= needed)) {
	    needed = (lines - linecounter);
	    if (sleepcounter == -1) {	//sometimes it scans a line short
		needed--;
		lines--;
		if (uimode != SANEUI) {
		    fspos = ftell(fd);
		    rewind(fd);
		    write_pnm_headers(fd, color, bytes_per_line, lines);
		    fseek(fd, fspos, SEEK_SET);
		} else {
		    write(ctloutpipe, &writelength, sizeof(writelength));
		    memset(data_array[0], 0, writelength);
		    written = write(datapipe, data_array[0], writelength);
		}
	    }
	}
	if ((color == COLOR) && ((3 * (lines - linecounter)) <= needed)) {
	    needed = 3 * (lines - linecounter);
	    if (sleepcounter == -1) {
		needed--;
		lines--;
		if (uimode != SANEUI) {
		    fspos = ftell(fd);
		    rewind(fd);
		    write_pnm_headers(fd, color, bytes_per_line, lines);
		    fseek(fd, fspos, SEEK_SET);
		} else {
		    write(ctloutpipe, &writelength, sizeof(writelength));
		    memset(data_array[0], 0, writelength);
		    written = write(datapipe, data_array[0], writelength);
		}
	    }
	}
	readylines = read_register(port, buffer_line_count_register);
	//printf("\n readylines = %d", readylines); fflush(stdout);
	//printf("\nneeded = %d",needed); fflush(stdout);        

	if (readylines < needed) {
	    usleep(50);
	    sleepcounter++;
	    if (sleepcounter > 1000) {
		if ((color == GREYSCALE)
		    && ((lines - linecounter) == needed))
		    sleepcounter = -1;
		if ((color == COLOR)
		    && ((3 * (lines - linecounter)) == needed))
		    sleepcounter = -1;
	    }
	    if (sleepcounter > 100000)
		exit(-1);

	} else {
	    for (neededcounter = 0; neededcounter < needed;
		 neededcounter++) {
		readbuf(port, redbuffer, bytes_per_line);
		if (color == COLOR) {
		    readbuf(port, greenbuffer, bytes_per_line);
		    readbuf(port, bluebuffer, bytes_per_line);
		}
		for (bytecounter = 0; bytecounter < bytes_per_line;
		     bytecounter++) {
		    if (color == GREYSCALE)
			data_array[neededcounter][bytecounter]
			    = gamma_table[bytecounter +
					  leftalign][(redbuffer
						      [bytecounter])];

		    if (color == COLOR) {
			data_array[neededcounter / 3][bytecounter * 3]
			    = gamma_table[(bytecounter + leftalign) *
					  3][(redbuffer[bytecounter])];

			data_array[neededcounter / 3][(bytecounter * 3) + 1]
			    = gamma_table[((bytecounter + leftalign) * 3) +
					  1][(greenbuffer[bytecounter])];

			data_array[neededcounter / 3][(bytecounter * 3) + 2]
			    = gamma_table[((bytecounter + leftalign) * 3) +
					2][(bluebuffer[bytecounter])];
		    }		//color
		}		//bytecounter
		if (color == COLOR)
		    neededcounter = neededcounter + 2;	//3 SRAM lines read
	    }			//neededcounter

	    inbuffer = needed;
	    if (color == COLOR)
		inbuffer = inbuffer / 3;
	    for (writecounter = 0; writecounter < inbuffer; writecounter++) {
		if ((color == GREYSCALE) && (lineart)) {
		    pack_lineart(data_array[writecounter], writelength);
		}
		if (uimode == SANEUI) {
		    if (!cancelled) {
			write(ctloutpipe, &writelength, sizeof(writelength));
			written =
			    write(datapipe, data_array[writecounter], writelength);
		    } else {
			writelength = 0;
			write(ctloutpipe, &writelength, sizeof(writelength));
			write_register(port, primary_register, MODE);	//end scan
			if (color != COLOR)
			    free_data_array(gamma_table, (2550 * dpi / 300));
			else
			    free_data_array(gamma_table, (3 * (2550 * dpi / 300)));
			free_data_array(data_array, needed);
//printf("\ncancelling scan...");  fflush(stdout);
			return;
		    }
//imagecounter +=written;
//printf("\nhave written = %d, total = %d",written,imagecounter); fflush(stdout);
		} else
		    fwrite(data_array[writecounter], writelength, 1, fd);	//write data to disk buffer
	    }
	    linecounter = linecounter + inbuffer;
	    if (pause != 0)
		usleep(5);	//for short lines, sometimes the computer
	    //reads faster than the scanner's controller
	    //can update the lines_in_register counter.
	    // if(sleepcounter != -1) sleepcounter = 0;
	}			//else (read buffers)
    }				//while linecounter

    write_register(port, primary_register, MODE);	//end scan
    if (uimode != SANEUI)
	fclose(fd);		//close the image file
    if (color != COLOR)
	free_data_array(gamma_table, (2550 * dpi / 300));
    else
	free_data_array(gamma_table, (3 * (2550 * dpi / 300)));
    free_data_array(data_array, needed);	//free the lookup table and scan buffer
    if (uimode != SANEUI)
	printf("\nScan Complete.\n");

}
